package com.dsic.gj_erp.service.zyjh;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jcsj.DmCbxxp;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.jhgk.DmYdjhLj;
import com.dsic.gj_erp.bean.zyjh.DmYppyz;
import com.dsic.gj_erp.controller.zyjh.dto.PgWithPcDto;
import com.dsic.gj_erp.controller.zyjh.dto.ShangLiao;
import com.dsic.gj_erp.dao.zyjh.PPBGongDto;
import com.dsic.gj_erp.mapper.jhgk.DmYdjhMapper;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.jcsj.DmCbxxpService;
import com.dsic.gj_erp.service.jhgk.DmYdjhLjService;
import com.google.common.collect.ImmutableMap;
import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class ZyjhService extends ServiceImpl<DmYdjhMapper,DmYdjh> {

    private final DmBzryService bzryService;
    private final DmCbxxpService dmCbxxpService;
    private final RedisTemplate<String,Object> redisTemplate;
    private final DmYdjhLjService ljService;
    private final DmYppyzService yppyzService;

    private static final String MPDHSJ_KEY="MPDHSJ";

    public void ppbg(Integer id){
        yppyzService.update(new UpdateWrapper<DmYppyz>().set("status","已配送").eq("id",id));
    }

    public List<DmYppyz> getYbf(DmYppyz yppyz) {
        return yppyzService.bgList(yppyz);
    }

    public void ppwc(Integer id,String dcCh,String dcPl,String dcFd){
        int count = ljService.count(Wrappers.<DmYdjhLj>lambdaQuery()
                .eq(DmYdjhLj::getCzbh, dcCh)
                .eq(DmYdjhLj::getPl, dcPl)
                .eq(DmYdjhLj::getFd, dcFd)
                .isNull(DmYdjhLj::getQq)
                .isNull(DmYdjhLj::getDw)
        );
        if (count==0){
            yppyzService.update(new UpdateWrapper<DmYppyz>().set("status","已摆放").eq("id",id));
        }
    }

    public void ppBGong(List<DmYdjhLj> list,String dcCh,String dcPl,String zyq){
        ljService.updateBatchById(list);
    }

    /**
     * 获取预配盘原则和对应未配盘零件信息
     */
    public Object getWppljAndPpyz(String zyq1){
        return baseMapper.getWppljAndPpyz(zyq1);
    }

    public Map<String,Object> getMPSYCLJLJBP(String dcCh,String tzbh){
        return baseMapper.getMPSYCLJLJBP(dcCh,tzbh);
    }

    public String getMpDhsjDhh(){
        String date=DateUtil.date().toString("yyMMdd");
        Object xh = redisTemplate.opsForHash().get(MPDHSJ_KEY, date);
        if(ObjUtil.isEmpty(xh)){
            redisTemplate.opsForHash().put(MPDHSJ_KEY, date,0);
        }
        Long increment = redisTemplate.opsForHash().increment(MPDHSJ_KEY, date, 1);
        return date+StrUtil.padPre(increment+"",3,"0");
    }

    public String getMpDhsjDhh(String date){
        Object xh = redisTemplate.opsForHash().get(MPDHSJ_KEY, date);
        if(ObjUtil.isEmpty(xh)){
            redisTemplate.opsForHash().put(MPDHSJ_KEY, date,0);
        }
        Long increment = redisTemplate.opsForHash().increment(MPDHSJ_KEY, date, 1);
        return date+StrUtil.padPre(increment+"",3,"0");
    }

    @Transactional(rollbackFor = Exception.class)
    public List<DmYdjhLj> createLj1(String dcCh, String dcPl, String tzbh){
        ljService.remove(Wrappers.<DmYdjhLj>lambdaQuery()
                .eq(DmYdjhLj::getCzbh,dcCh)
                .eq(DmYdjhLj::getPl,dcPl)
                .eq(DmYdjhLj::getTlth,tzbh)
        );
        List<DmYdjhLj> ljInfo = baseMapper.getLjInfo(dcCh, dcPl, tzbh);
        ljService.saveBatch(ljInfo);
        return ljInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    public List<DmYdjhLj> createLj(String dcCh, String dcPl, String tzbh){
        ljService.remove(Wrappers.<DmYdjhLj>lambdaQuery()
                .eq(DmYdjhLj::getCzbh,dcCh)
                .eq(DmYdjhLj::getPl,dcPl)
                .eq(DmYdjhLj::getTlth,tzbh)
        );
        List<DmYdjhLj> ljInfo = baseMapper.getLjInfo(dcCh, dcPl, tzbh);
        ljService.saveBatch(ljInfo);
        return ljInfo;
    }

    public List<DmYdjh> getYdjhByZtOrForeman(Integer zt,String zyq){
        return baseMapper.getWithDjh(zt,zyq);
    }

    public List<DmYdjh> getYdjhByZtOrForeman(Integer zt,String foreman,String userCode,String zyq){
        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<DmYdjh>()
                .ge("zt",11)
                .ge(ObjUtil.isNotEmpty(zt),"zt",zt)
                .eq(StrUtil.isNotEmpty(zyq),"zyq1",zyq);

        switch (zt){
            case 31:
//                wrapper.eq(!"1".equals(foreman),"pwpgry",foreman);
//                wrapper.eq("1".equals(foreman),"pwpgry",userCode);
                wrapper.isNull("pwfkry");
                break;
            case 51:
//                wrapper.eq(!"1".equals(foreman),"hxpgry",foreman);
//                wrapper.eq("1".equals(foreman),"hxpgry",userCode);
                wrapper.isNull("hxfkry");
                break;
            case 61:
//                wrapper.eq(!"1".equals(foreman),"qgpgry",foreman);
//                wrapper.eq("1".equals(foreman),"qgpgry",userCode);
                wrapper.isNull("qgfkry");
                break;
        }
        wrapper.orderByDesc("dc_ch","dc_pl,tzbh");

        return list(wrapper);
    }

    public List<Map<String,Object>> getYdjhByZt(Integer zt,String wgzt,String wclzt,String zyq){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.getYdjhByZt(zyq,zt,wgzt,wclzt,from,to);
    }


    public void pgWithForeman(PgWithPcDto dto){
        LambdaUpdateWrapper<DmYdjh> wrapper = new LambdaUpdateWrapper<DmYdjh>()
                .eq(DmYdjh::getDcCh,dto.getDcCh())
                .eq(DmYdjh::getDcPl,dto.getDcPl())
                .eq(DmYdjh::getZyq1,dto.getZyq());

        String pgrq = DateUtil.date().toString("yyyy/MM/dd HH:mm:ss");
        switch (dto.getZt()){
            case "21":
                wrapper.set(DmYdjh::getPwpgrq,pgrq).set(DmYdjh::getPwpgry,dto.getForemanId());
                wrapper.lt(DmYdjh::getZt,31);//仅处理未派工的数据
                break;
            case "42":
            case "52":
                wrapper.ge(DmYdjh::getZt,32);
                if ("HXPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getHxpgrq,pgrq).set(DmYdjh::getHxpgry,dto.getForemanId()).set(DmYdjh::getZt,"51");
                    wrapper.isNull(DmYdjh::getHxpgrq).isNull(DmYdjh::getHxpgry);
                    //wrapper.lt(DmYdjh::getZt,51);//仅处理未派工的数据
                }
                if ("QGPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getQgpgrq,pgrq).set(DmYdjh::getQgpgry,dto.getForemanId()).set(DmYdjh::getZt,"61");
                    wrapper.eq(DmYdjh::getZt,"52").isNull(DmYdjh::getQgpgrq).isNull(DmYdjh::getQgpgry);
                    //wrapper.lt(DmYdjh::getZt,61);//仅处理未派工的数据
                }
                break;
        }

        this.update(wrapper);
    }

    public DmBzry getForemanByWorker(String worker){
        DmBzry one = bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, worker));
        if ("1".equals(one.getForeman())){
            return one;
        }
        return bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, one.getForeman()));
    }

    public List<DmBzry> getForeman(String zyq){
        return bzryService.list(Wrappers.<DmBzry>lambdaQuery()
                .eq(DmBzry::getForeman, "1").eq(DmBzry::getZyq, zyq));
    }

    public List<DmBzry> getBzryByBz(String zyq,String bz){
        return bzryService.list(Wrappers.<DmBzry>lambdaQuery()
                .eq(DmBzry::getResourcesCode, bz).eq(DmBzry::getZyq, zyq).orderByAsc(DmBzry::getUserName));
    }
    /**
     * 获取所有未处理批次
     */
    public List<Map<String,Object>> getWcl(){
        return baseMapper.getWcl();
    }

    public List<Map<String, Object>> gcjd(){
        ImmutableMap<String, Boolean> map = ImmutableMap.of("flag", true);
        List<DmCbxxp> list = dmCbxxpService.getList(map);
        List<String> czbhList=list.stream().map(DmCbxxp::getCbmc).collect(Collectors.toList());
        return baseMapper.gcjd(czbhList);
    }

    public List<Map<String, Object>> gxjhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gxjhwcqk(from,to);
    }

    public List<Map<String, Object>> gyfcl(){
        return baseMapper.gyfcl();
    }

    public List<Map<String, Object>> jhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.jhwcqk(from,to);
    }

    public Map<String,Object> gekuachaliang(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gekuachaliang(from,to,"11");
    }

    public void updateForSl(ShangLiao sl){
        String date=DateUtil.date().toString("yyyy/MM/dd");
        LambdaUpdateWrapper<DmYdjh> wrapper = Wrappers.<DmYdjh>lambdaUpdate()
                //上料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlycldw()),DmYdjh::getSlycldw, sl.getYdjh().getSlycldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlyclcs()),DmYdjh::getSlyclcs, sl.getYdjh().getSlyclcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkry, sl.getYdjh().getSlfkry())

                //理料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLldw()),DmYdjh::getLldw, sl.getYdjh().getLldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlcs()),DmYdjh::getLlcs, sl.getYdjh().getLlcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkry, sl.getYdjh().getLlfkry())
                .eq(DmYdjh::getId, sl.getYdjh().getId());

        switch (sl.getType()){
            case YCL:
                wrapper.set(DmYdjh::getZt, DmYdjh.getSlZt());
                break;
            case LLJ:
                wrapper.set(DmYdjh::getZt, DmYdjh.getLLZt());
        }


        this.update(wrapper);
    }

    private static final String KEY="gxControl";

    public Object cacheable(String gx){
        if (gx!=null){
            redisTemplate.opsForValue().set(KEY,gx);
            return gx;
        }else{
            return redisTemplate.opsForValue().get(KEY);
        }
    }
}
